"""
import all wrapper function
"""

# callback
from je_auto_control.utils.callback.callback_function_executor import \
    callback_executor
# Critical
from je_auto_control.utils.critical_exit.critcal_exit import CriticalExit
from je_auto_control.utils.cv2_utils.screen_record import ScreenRecorder
# utils cv2_utils
from je_auto_control.utils.cv2_utils.screenshot import pil_screenshot
from je_auto_control.utils.exception.exceptions import \
    AutoControlActionException
from je_auto_control.utils.exception.exceptions import \
    AutoControlActionNullException
from je_auto_control.utils.exception.exceptions import \
    AutoControlCantFindKeyException
# Exception
from je_auto_control.utils.exception.exceptions import AutoControlException
from je_auto_control.utils.exception.exceptions import \
    AutoControlJsonActionException
from je_auto_control.utils.exception.exceptions import \
    AutoControlKeyboardException
from je_auto_control.utils.exception.exceptions import \
    AutoControlMouseException
from je_auto_control.utils.exception.exceptions import \
    AutoControlRecordException
from je_auto_control.utils.exception.exceptions import \
    AutoControlScreenException
from je_auto_control.utils.exception.exceptions import ImageNotFoundException
from je_auto_control.utils.executor.action_executor import \
    add_command_to_executor
# executor
from je_auto_control.utils.executor.action_executor import execute_action
from je_auto_control.utils.executor.action_executor import execute_files
from je_auto_control.utils.executor.action_executor import executor
# file process
from je_auto_control.utils.file_process.get_dir_file_list import \
    get_dir_files_as_list
# html report
from je_auto_control.utils.generate_report.generate_html_report import \
    generate_html
from je_auto_control.utils.generate_report.generate_html_report import \
    generate_html_report
from je_auto_control.utils.generate_report.generate_json_report import \
    generate_json
from je_auto_control.utils.generate_report.generate_json_report import \
    generate_json_report
# xml
from je_auto_control.utils.generate_report.generate_xml_report import \
    generate_xml
from je_auto_control.utils.generate_report.generate_xml_report import \
    generate_xml_report
# json
from je_auto_control.utils.json.json_file import read_action_json
from je_auto_control.utils.json.json_file import write_action_json
# package manager
from je_auto_control.utils.package_manager.package_manager_class import \
    package_manager
from je_auto_control.utils.project.create_project_structure import \
    create_project_dir
# Shell command
from je_auto_control.utils.shell_process.shell_exec import ShellManager
from je_auto_control.utils.shell_process.shell_exec import default_shell_manager
# socket server
from je_auto_control.utils.socket_server.auto_control_socket_server import \
    start_autocontrol_socket_server
# Start exe
from je_auto_control.utils.start_exe.start_another_process import start_exe
# test record
from je_auto_control.utils.test_record.record_test_class import \
    test_record_instance
# timeout
from je_auto_control.utils.timeout.multiprocess_timeout import \
    multiprocess_timeout
from je_auto_control.wrapper.auto_control_image import locate_all_image
from je_auto_control.wrapper.auto_control_image import locate_and_click
from je_auto_control.wrapper.auto_control_image import locate_image_center
# import keyboard
from je_auto_control.wrapper.auto_control_keyboard import check_key_is_press
from je_auto_control.wrapper.auto_control_keyboard import get_keyboard_keys_table
from je_auto_control.wrapper.auto_control_keyboard import get_special_table
from je_auto_control.wrapper.auto_control_keyboard import hotkey
from je_auto_control.wrapper.auto_control_keyboard import keyboard_keys_table
from je_auto_control.wrapper.auto_control_keyboard import press_keyboard_key
from je_auto_control.wrapper.auto_control_keyboard import release_keyboard_key
from je_auto_control.wrapper.auto_control_keyboard import type_keyboard
from je_auto_control.wrapper.auto_control_keyboard import write
# import mouse
from je_auto_control.wrapper.auto_control_mouse import click_mouse
from je_auto_control.wrapper.auto_control_mouse import get_mouse_position
from je_auto_control.wrapper.auto_control_mouse import mouse_keys_table
from je_auto_control.wrapper.auto_control_mouse import mouse_scroll
from je_auto_control.wrapper.auto_control_mouse import press_mouse
from je_auto_control.wrapper.auto_control_mouse import release_mouse
from je_auto_control.wrapper.auto_control_mouse import set_mouse_position
from je_auto_control.wrapper.auto_control_mouse import special_mouse_keys_table
# test_record
from je_auto_control.wrapper.auto_control_record import record
from je_auto_control.wrapper.auto_control_record import stop_record
# import screen
from je_auto_control.wrapper.auto_control_screen import screen_size
from je_auto_control.wrapper.auto_control_screen import screenshot
# Recording
from je_auto_control.utils.cv2_utils.video_recording import RecordingThread

__all__ = [
    "click_mouse", "mouse_keys_table", "get_mouse_position", "press_mouse", "release_mouse",
    "mouse_scroll", "set_mouse_position", "special_mouse_keys_table",
    "keyboard_keys_table", "press_keyboard_key", "release_keyboard_key", "type_keyboard", "check_key_is_press",
    "write", "hotkey", "start_exe", "get_keyboard_keys_table",
    "screen_size", "screenshot", "locate_all_image", "locate_image_center", "locate_and_click",
    "CriticalExit", "AutoControlException", "AutoControlKeyboardException",
    "AutoControlMouseException", "AutoControlCantFindKeyException",
    "AutoControlScreenException", "ImageNotFoundException", "AutoControlJsonActionException",
    "AutoControlRecordException", "AutoControlActionNullException", "AutoControlActionException", "record",
    "stop_record", "read_action_json", "write_action_json", "execute_action", "execute_files", "executor",
    "add_command_to_executor", "multiprocess_timeout", "test_record_instance", "screenshot", "pil_screenshot",
    "generate_html", "generate_html_report", "generate_json", "generate_json_report", "generate_xml",
    "generate_xml_report", "get_dir_files_as_list", "create_project_dir", "start_autocontrol_socket_server",
    "callback_executor", "package_manager", "get_special_table", "ShellManager", "default_shell_manager",
    "RecordingThread"
]



# general
class AutoControlException(Exception):
    pass


# Keyboard
class AutoControlKeyboardException(Exception):
    pass


class AutoControlCantFindKeyException(Exception):
    pass


# Mouse
class AutoControlMouseException(Exception):
    pass


# Screen


class AutoControlScreenException(Exception):
    pass


# Image detect


class ImageNotFoundException(Exception):
    pass


# Record


class AutoControlRecordException(Exception):
    pass


# Execute action

class AutoControlExecuteActionException(Exception):
    pass


class AutoControlJsonActionException(Exception):
    pass


class AutoControlActionNullException(Exception):
    pass


class AutoControlActionException(Exception):
    pass


class AutoControlAddCommandException(Exception):
    pass


class AutoControlArgparseException(Exception):
    pass


# timeout
class AutoControlTimeoutException(Exception):
    pass


# html exception

class AutoControlHTMLException(Exception):
    pass


# Json Exception

class AutoControlJsonException(Exception):
    pass


class AutoControlGenerateJsonReportException(Exception):
    pass


# XML

class XMLException(Exception):
    pass


class XMLTypeException(Exception):
    pass


# Execute callback
class CallbackExecutorException(Exception):
    pass



import sys
import time

from je_auto_control.utils.exception.exception_tags import linux_import_error
from je_auto_control.utils.exception.exceptions import AutoControlException

if sys.platform not in ["linux", "linux2"]:
    raise AutoControlException(linux_import_error)

from je_auto_control.linux_with_x11.core.utils.x11_linux_display import display
from Xlib.ext.xtest import fake_input
from Xlib import X


def press_key(keycode: int) -> None:
    """
    :param keycode which keycode we want to press
    """
    time.sleep(0.01)
    fake_input(display, X.KeyPress, keycode)
    display.sync()


def release_key(keycode: int) -> None:
    """
    :param keycode which keycode we want to release
    """
    time.sleep(0.01)
    fake_input(display, X.KeyRelease, keycode)
    display.sync()



import threading

import cv2
import numpy as np
from mss import mss

from je_auto_control.utils.logging.loggin_instance import autocontrol_logger


class RecordingThread(threading.Thread):

    def __init__(self, video_name: str = "autocontrol_recoding"):
        autocontrol_logger.info("Init RecordingThread")
        super().__init__()
        self.recoding_flag = True
        self.video_name = video_name
        self.daemon = True
        self.fps = 20

    def set_recoding_flag(self, recoding_flag: bool):
        autocontrol_logger.info(f"RecordingThread set_recoding_flag recoding_flag: {recoding_flag}")
        self.recoding_flag = recoding_flag

    def run(self):
        with mss() as sct:
            resolution = sct.monitors[0]
            self.video_name = self.video_name + '.mp4'
            fourcc = cv2.VideoWriter_fourcc(*'mp4v')
            video_writer = cv2.VideoWriter(self.video_name, fourcc, self.fps,
                                           (resolution['width'], resolution['height']))
            while self.recoding_flag:
                screen_image = sct.grab(resolution)
                image_rgb = cv2.cvtColor(np.array(screen_image), cv2.COLOR_BGRA2BGR)
                video_writer.write(image_rgb)
            else:
                video_writer.release()



traditional_chinese_word_dict = {
    # Main
    "application_name": "AutoControlGUI",
    # Widget
    "interval_time": "間隔時間（秒）：",
    "cursor_x": "滑鼠游標 X 軸位置：",
    "cursor_y": "滑鼠游標 Y 軸位置：",
    "mouse_button": "滑鼠按鍵：",
    "keyboard_button": "鍵盤按鍵：",
    "click_type": "點擊類型：",
    "input_method": "輸入方式：",
    "mouse_radio": "滑鼠",
    "keyboard_radio": "鍵盤",
    "repeat_until_stopped_radio": "重複直到停止",
    "repeat_radio": "重複",
    "times": "次數",
    "start": "開始",
    "stop": "停止",
}



from ctypes import WINFUNCTYPE, c_bool, c_int, POINTER, create_unicode_buffer
from typing import Union

from je_auto_control.windows.core.utils.win32_ctype_input import user32

EnumWindows = user32.EnumWindows
EnumWindowsProc = WINFUNCTYPE(c_bool, POINTER(c_int), POINTER(c_int))
GetWindowText = user32.GetWindowTextW
GetWindowTextLength = user32.GetWindowTextLengthW
IsWindowVisible = user32.IsWindowVisible
FindWindowW = user32.FindWindowW
CloseWindow = user32.CloseWindow
DestroyWindow = user32.DestroyWindow


def get_all_window_hwnd():
    window_info = []

    def _foreach_window(hwnd, l_param):
        if IsWindowVisible(hwnd):
            length = GetWindowTextLength(hwnd)
            buff = create_unicode_buffer(length + 1)
            GetWindowText(hwnd, buff, length + 1)
            window_info.append((hwnd, buff.value))
        return True

    EnumWindows(EnumWindowsProc(_foreach_window), 0)
    return window_info


def get_one_window_hwnd(window_class: Union[None, str], window_name: Union[None, str]):
    return FindWindowW(window_class, window_name)


def close_window(hwnd) -> bool:
    return CloseWindow(hwnd)


def destroy_window(hwnd) -> bool:
    return DestroyWindow(hwnd)



# error tags
je_auto_control_error: str = "Auto-control error"
je_auto_control_critical_exit_error: str = "Auto-control critical exit error"

# os tags
linux_import_error: str = "Should only be loaded on Linux"
osx_import_error: str = "Should only be loaded on macOS"
windows_import_error: str = "Should only be loaded on Windows"
macos_record_error: str = "Cannot use recorder on macOS"

# keyboard tags
keyboard_error: str = "Auto-control keyboard error"
keyboard_press_key: str = "Keyboard key press error"
keyboard_release_key: str = "Keyboard key release error"
keyboard_type_key: str = "Keyboard key type error"
keyboard_write: str = "Keyboard write error"
keyboard_write_cant_find: str = "Keyboard write error: key not found"
keyboard_hotkey: str = "Keyboard hotkey error"

# mouse tags
mouse_error: str = "Auto-control mouse error"
mouse_get_position: str = "Mouse position retrieval error"
mouse_set_position: str = "Mouse position set error"
mouse_press_mouse: str = "Mouse press error"
mouse_release_mouse: str = "Mouse release error"
mouse_click_mouse: str = "Mouse click error"
mouse_scroll: str = "Mouse scroll error"
mouse_wrong_value: str = "Mouse value error"

# screen tags
screen_error: str = "Auto-control screen error"
screen_get_size: str = "Screen size retrieval error"
screen_screenshot: str = "Screen screenshot error"

# table tags
table_cant_find_key: str = "Cannot find key error"

# cv2_utils tags
cant_find_image: str = "Cannot find image"
find_image_error_variable: str = "Variable error"

# listener tags
listener_error: str = "Auto-control listener error"

# test_record tags
record_queue_error: str = "Cannot get test_record queue: it is None. Are you stopping test_record before running it?"
record_not_found_action_error: str = "test_record action not found"

# json tag
cant_execute_action_error: str = "Cannot execute action"
cant_generate_json_report: str = "Cannot generate JSON report"
cant_find_json_error: str = "Cannot find JSON file"
cant_save_json_error: str = "Cannot save JSON file"
action_is_null_error: str = "JSON action is null"

# timeout tag
timeout_need_on_main_error: str = "Timeout function must be in main"

# HTML
html_generate_no_data_tag: str = "Record is None"

# add command
add_command_exception: str = "Command value must be a method or function"

# executor
executor_list_error: str = "Executor received invalid data: list is None or wrong type"

# argparse
argparse_get_wrong_data: str = "Argparse received invalid data"

# XML
cant_read_xml_error: str = "Cannot read XML"
xml_type_error: str = "XML type error"

# Callback executor
get_bad_trigger_method: str = "Invalid trigger method: only kwargs and args accepted"
get_bad_trigger_function: str = "Invalid trigger function: only functions in event_dict accepted"

# Can't find file
can_not_find_file: str = "Cannot find file"


import sys

from PySide6.QtWidgets import QMainWindow, QApplication
from qt_material import QtStyleTools

from je_auto_control.gui.language_wrapper.multi_language_wrapper import language_wrapper
from je_auto_control.gui.main_widget import AutoControlGUIWidget


class AutoControlGUIUI(QMainWindow, QtStyleTools):

    def __init__(self):
        super().__init__()
        self.id = language_wrapper.language_word_dict.get("application_name")
        if sys.platform in ["win32", "cygwin", "msys"]:
            from ctypes import windll
            windll.shell32.SetCurrentProcessExplicitAppUserModelID(self.id)
        self.setStyleSheet(
            f"font-size: 12pt;"
            f"font-family: 'Lato';"
        )
        self.apply_stylesheet(self, "dark_amber.xml")
        self.auto_control_gui_widget = AutoControlGUIWidget()
        self.setCentralWidget(self.auto_control_gui_widget)



import xml.dom.minidom
from xml.etree import ElementTree

from je_auto_control.utils.exception.exception_tags import cant_read_xml_error
from je_auto_control.utils.exception.exception_tags import xml_type_error
from je_auto_control.utils.exception.exceptions import XMLException
from je_auto_control.utils.exception.exceptions import XMLTypeException


def reformat_xml_file(xml_string: str):
    dom = xml.dom.minidom.parseString(xml_string)
    return dom.toprettyxml()


class XMLParser(object):

    def __init__(self, xml_string: str, xml_type: str = "string"):
        """
        :param xml_string: full xml string
        :param xml_type: file or string
        """
        self.element_tree = ElementTree
        self.tree = None
        self.xml_root = None
        self.xml_from_type = "string"
        self.xml_string = xml_string.strip()
        xml_type = xml_type.lower()
        if xml_type not in ["file", "string"]:
            raise XMLTypeException(xml_type_error)
        if xml_type == "string":
            self.xml_parser_from_string()
        else:
            self.xml_parser_from_file()

    def xml_parser_from_string(self, **kwargs):
        """
        :param kwargs: any another param
        :return: xml root element tree
        """
        try:
            self.xml_root = ElementTree.fromstring(self.xml_string, **kwargs)
        except XMLException:
            raise XMLException(cant_read_xml_error)
        return self.xml_root

    def xml_parser_from_file(self, **kwargs):
        """
        :param kwargs: any another param
        :return: xml root element tree
        """
        try:
            self.tree = ElementTree.parse(self.xml_string, **kwargs)
        except XMLException:
            raise XMLException(cant_read_xml_error)
        self.xml_root = self.tree.getroot()
        self.xml_from_type = "file"
        return self.xml_root

    def write_xml(self, write_xml_filename: str, write_content: str):
        """
        :param write_xml_filename:  xml file name
        :param write_content: content to write
        """
        write_content = write_content.strip()
        content = self.element_tree.fromstring(write_content)
        tree = self.element_tree.ElementTree(content)
        tree.write(write_xml_filename, encoding="utf-8")



from PySide6.QtCore import QTimer
from PySide6.QtGui import QIntValidator, QKeyEvent, Qt
from PySide6.QtWidgets import (
    QWidget, QLineEdit, QComboBox, QPushButton, QVBoxLayout, QLabel,
    QGridLayout, QHBoxLayout, QRadioButton, QButtonGroup, QMessageBox
)

from je_auto_control.gui.language_wrapper.multi_language_wrapper import language_wrapper
from je_auto_control.utils.executor.action_executor import execute_action
from je_auto_control.wrapper.auto_control_keyboard import type_keyboard
from je_auto_control.wrapper.auto_control_mouse import click_mouse
from je_auto_control.wrapper.auto_control_record import record, stop_record
from je_auto_control.wrapper.platform_wrapper import keyboard_keys_table, mouse_keys_table


class AutoControlGUIWidget(QWidget):

    def __init__(self, parent=None):
        super().__init__(parent)

        main_layout = QVBoxLayout()

        # Grid for input fields
        grid = QGridLayout()

        # Interval time
        grid.addWidget(QLabel(language_wrapper.language_word_dict.get("interval_time")), 0, 0)
        self.interval_input = QLineEdit()
        self.interval_input.setValidator(QIntValidator())
        grid.addWidget(self.interval_input, 0, 1)

        # Cursor X/Y
        grid.addWidget(QLabel(language_wrapper.language_word_dict.get("cursor_x")), 2, 0)
        self.cursor_x_input = QLineEdit()
        self.cursor_x_input.setValidator(QIntValidator())
        grid.addWidget(self.cursor_x_input, 2, 1)

        grid.addWidget(QLabel(language_wrapper.language_word_dict.get("cursor_y")), 3, 0)
        self.cursor_y_input = QLineEdit()
        self.cursor_y_input.setValidator(QIntValidator())
        grid.addWidget(self.cursor_y_input, 3, 1)

        # Mouse button
        grid.addWidget(QLabel(language_wrapper.language_word_dict.get("mouse_button")), 4, 0)
        self.mouse_button_combo = QComboBox()
        self.mouse_button_combo.addItems(mouse_keys_table)
        grid.addWidget(self.mouse_button_combo, 4, 1)

        # Keyboard button
        grid.addWidget(QLabel(language_wrapper.language_word_dict.get("keyboard_button")), 5, 0)
        self.keyboard_button_combo = QComboBox()
        self.keyboard_button_combo.addItems(keyboard_keys_table.keys())
        grid.addWidget(self.keyboard_button_combo, 5, 1)

        # Click type
        grid.addWidget(QLabel(language_wrapper.language_word_dict.get("click_type")), 6, 0)
        self.click_type_combo = QComboBox()
        self.click_type_combo.addItems(["Single Click", "Double Click"])
        grid.addWidget(self.click_type_combo, 6, 1)

        # Input method selection
        grid.addWidget(QLabel(language_wrapper.language_word_dict.get("input_method")), 7, 0)
        self.mouse_radio = QRadioButton(language_wrapper.language_word_dict.get("mouse_radio"))
        self.keyboard_radio = QRadioButton(language_wrapper.language_word_dict.get("keyboard_radio"))
        self.mouse_radio.setChecked(True)
        self.input_method_group = QButtonGroup()
        self.input_method_group.addButton(self.mouse_radio)
        self.input_method_group.addButton(self.keyboard_radio)
        grid.addWidget(self.mouse_radio, 7, 1)
        grid.addWidget(self.keyboard_radio, 7, 2)

        main_layout.addLayout(grid)

        # Repeat options
        repeat_layout = QHBoxLayout()
        self.repeat_until_stopped = QRadioButton(language_wrapper.language_word_dict.get("repeat_until_stopped_radio"))
        self.repeat_count_times = QRadioButton(language_wrapper.language_word_dict.get("repeat_radio"))
        self.repeat_count_input = QLineEdit()
        self.repeat_count_input.setValidator(QIntValidator())
        self.repeat_count_input.setPlaceholderText(language_wrapper.language_word_dict.get("times"))
        repeat_group = QButtonGroup()
        repeat_group.addButton(self.repeat_until_stopped)
        repeat_group.addButton(self.repeat_count_times)
        self.repeat_until_stopped.setChecked(True)
        self.repeat_count = 0
        self.repeat_max = 0

        repeat_layout.addWidget(self.repeat_until_stopped)
        repeat_layout.addWidget(self.repeat_count_times)
        repeat_layout.addWidget(self.repeat_count_input)
        main_layout.addLayout(repeat_layout)

        # Start/Stop buttons
        button_layout = QHBoxLayout()
        self.start_button = QPushButton(language_wrapper.language_word_dict.get("start"))
        self.start_button.clicked.connect(self.start_autocontrol)
        self.stop_button = QPushButton(language_wrapper.language_word_dict.get("stop"))
        self.stop_button.clicked.connect(self.stop_autocontrol)
        button_layout.addWidget(self.start_button)
        button_layout.addWidget(self.stop_button)
        main_layout.addLayout(button_layout)

        # Timer
        self.start_autocontrol_timer = QTimer()

        # Connect input method toggle
        self.mouse_radio.toggled.connect(self.update_input_mode)
        self.keyboard_radio.toggled.connect(self.update_input_mode)
        self.update_input_mode()

        self.setLayout(main_layout)

    def update_input_mode(self):
        use_mouse = self.mouse_radio.isChecked()
        self.cursor_x_input.setEnabled(use_mouse)
        self.cursor_y_input.setEnabled(use_mouse)
        self.mouse_button_combo.setEnabled(use_mouse)
        self.keyboard_button_combo.setEnabled(not use_mouse)

    def start_autocontrol(self):
        self.start_autocontrol_timer.setInterval(int(self.interval_input.text()))
        self.start_autocontrol_timer.timeout.connect(lambda: self.start_timer_function())
        self.start_autocontrol_timer.start()
        self.repeat_max = int(self.repeat_count_input.text())

    def start_timer_function(self):
        if self.repeat_until_stopped.isChecked():
            self.trigger_autocontrol_function()
        elif self.repeat_count_times.isChecked():
            self.repeat_count += 1
            if self.repeat_count < self.repeat_max:
                self.trigger_autocontrol_function()
            else:
                self.repeat_count = 0
                self.repeat_max = 0
                self.start_autocontrol_timer.stop()

    def trigger_autocontrol_function(self):
        click_type = self.click_type_combo.currentText()
        if self.mouse_radio.isChecked():
            trigger_function = click_mouse
            button = self.mouse_button_combo.currentText()
            x = int(self.cursor_x_input.text())
            y = int(self.cursor_y_input.text())
            if click_type == "Single Click":
                trigger_function(mouse_keycode=button, x=x, y=y)
            elif click_type == "Double Click":
                trigger_function(mouse_keycode=button, x=x, y=y)
                trigger_function(mouse_keycode=button, x=x, y=y)
        elif self.keyboard_radio.isChecked():
            trigger_function = type_keyboard
            button = self.keyboard_button_combo.currentText()
            if click_type == "Single Click":
                trigger_function(keycode=button)
            elif click_type == "Double Click":
                trigger_function(keycode=button)
                trigger_function(keycode=button)

    def stop_autocontrol(self):
        self.start_autocontrol_timer.stop()


    def keyPressEvent(self, event: QKeyEvent):
        if event.modifiers() == Qt.KeyboardModifier.ControlModifier and event.key() == Qt.Key.Key_4:
            self.start_autocontrol_timer.stop()
        else:
            super().keyPressEvent(event)



import sys

from je_auto_control.utils.exception.exception_tags import osx_import_error
from je_auto_control.utils.exception.exceptions import AutoControlException

if sys.platform not in ["darwin"]:
    raise AutoControlException(osx_import_error)

import Quartz


def check_key_is_press(keycode: int) -> bool:
    """
    :param keycode which keycode we want to check
    """
    return Quartz.CGEventSourceKeyState(0, keycode)



import sys

from je_auto_control.utils.exception.exception_tags import windows_import_error
from je_auto_control.utils.exception.exceptions import AutoControlException

if sys.platform not in ["win32", "cygwin", "msys"]:
    raise AutoControlException(windows_import_error)

import ctypes
from ctypes import wintypes
from je_auto_control.windows.core.utils.win32_vk import win32_EventF_UNICODE, win32_VkToVSC

user32 = ctypes.WinDLL('user32', use_last_error=True)

wintypes.ULONG_PTR = wintypes.WPARAM

Mouse: int = 0
Keyboard: int = 1
Hardware: int = 2


class MouseInput(ctypes.Structure):
    _fields_: tuple = (("dx", wintypes.LONG),
                       ("dy", wintypes.LONG),
                       ("mouseData", wintypes.DWORD),
                       ("dwFlags", wintypes.DWORD),
                       ("time", wintypes.DWORD),
                       ("dwExtraInfo", ctypes.c_void_p))


class KeyboardInput(ctypes.Structure):
    _fields_: tuple = (("wVk", wintypes.WORD),
                       ("wScan", wintypes.WORD),
                       ("dwFlags", wintypes.DWORD),
                       ("time", wintypes.DWORD),
                       ("dwExtraInfo", ctypes.c_void_p))

    def __init__(self, *args, **kwds):
        super(KeyboardInput, self).__init__(*args, **kwds)
        if not self.dwFlags & win32_EventF_UNICODE:
            self.wScan = user32.MapVirtualKeyExW(self.wVk, win32_VkToVSC, 0)


class HardwareInput(ctypes.Structure):
    _fields_: tuple = (("uMsg", wintypes.DWORD),
                       ("wParamL", wintypes.WORD),
                       ("wParamH", wintypes.WORD))


class Input(ctypes.Structure):
    class INPUTUnion(ctypes.Union):
        _fields_: tuple = (("ki", KeyboardInput),
                           ("mi", MouseInput),
                           ("hi", HardwareInput))

    _anonymous_: tuple = ("_input",)
    _fields_: tuple = (("type", wintypes.DWORD),
                       ("_input", INPUTUnion))


def _check_count(result, func, args) -> list:
    if result == 0:
        raise ctypes.WinError(ctypes.get_last_error())
    return args


LPINPUT: ctypes.POINTER = ctypes.POINTER(Input)

SendInput: user32.SendInput = user32.SendInput

user32.SendInput.errcheck = _check_count
user32.SendInput.arg_types = (wintypes.UINT, ctypes.c_void_p, ctypes.c_int)



from typing import Tuple, List

import cv2
import numpy as np

from je_auto_control.utils.cv2_utils.screenshot import pil_screenshot
from je_auto_control.utils.exception.exception_tags import screen_get_size
from je_auto_control.utils.exception.exception_tags import screen_screenshot
from je_auto_control.utils.exception.exceptions import AutoControlScreenException
from je_auto_control.utils.logging.loggin_instance import autocontrol_logger
from je_auto_control.utils.test_record.record_test_class import record_action_to_list
from je_auto_control.wrapper.platform_wrapper import screen


def screen_size() -> Tuple[int, int]:
    """
    get screen size
    """
    autocontrol_logger.info("AC_screen_size")
    try:
        try:
            record_action_to_list("size", None)
            return screen.size()
        except AutoControlScreenException:
            autocontrol_logger.error(f"screen_size, failed: {repr(AutoControlScreenException(screen_get_size))}")
            raise AutoControlScreenException(screen_get_size)
    except Exception as error:
        record_action_to_list("size", None, repr(error))
        autocontrol_logger.error(f"screen_size, failed: {repr(error)}")


def screenshot(file_path: str = None, screen_region: list = None) -> List[int]:
    """
    use to capture current screen cv2_utils
    :param file_path screenshot file save path
    :param screen_region screenshot screen_region
    """
    autocontrol_logger.info(f"screen_size, file_path: {file_path}, screen_region: {screen_region}")
    param = locals()
    try:
        try:
            record_action_to_list("AC_screenshot", param)
            return cv2.cvtColor(
                np.array(pil_screenshot(file_path=file_path, screen_region=screen_region)), cv2.COLOR_RGB2BGR)
        except AutoControlScreenException as error:
            autocontrol_logger.info(
                f"screen_size, file_path: {file_path}, screen_region: {screen_region}, "
                f"failed: {AutoControlScreenException(screen_screenshot + ' ' + repr(error))}")
            raise AutoControlScreenException(screen_screenshot + " " + repr(error))
    except Exception as error:
        record_action_to_list("AC_screenshot", None, repr(error))
        autocontrol_logger.info(
            f"screen_size, file_path: {file_path}, screen_region: {screen_region}, "
            f"failed: {repr(error)}")



from je_auto_control.windows.core.utils.win32_ctype_input import user32
from je_auto_control.windows.window.window_hwnd import FindWindowW

PostMessageW = user32.PostMessageW
SendMessageW = user32.SendMessageW

messages = {
    "WM_ACTIVATEAPP": 0x001C,
    "WM_CANCELMODE": 0x001F,
    "WM_CHILDACTIVATE": 0x0022,
    "WM_CLOSE": 0x0010,
    "WM_COMPACTING": 0x0041,
    "WM_CREATE": 0x0001,
    "WM_DESTROY": 0x0002,
    "WM_ENABLE": 0x000A,
    "WM_ENTERSIZEMOVE": 0x0231,
    "WM_EXITSIZEMOVE": 0x0232,
    "WM_GETICON": 0x007F,
    "WM_GETMINMAXINFO": 0x0024,
    "WM_INPUTLANGCHANGE": 0x0051,
    "WM_INPUTLANGCHANGEREQUEST": 0x0050,
    "WM_MOVE": 0x0003,
    "WM_MOVING": 0x0216,
    "WM_NCACTIVATE": 0x0086,
    "WM_NCCALCSIZE": 0x0083,
    "WM_NCCREATE": 0x0081,
    "WM_NCDESTROY": 0x0082,
    "WM_NULL": 0x0000,
    "WM_QUERYDRAGICON": 0x0037,
    "WM_QUERYOPEN": 0x0013,
    "WM_QUIT": 0x0012,
    "WM_SHOWWINDOW": 0x0018,
    "WM_SIZE": 0x0005,
    "WM_SIZING": 0x0214,
    "WM_STYLECHANGED": 0x007D,
    "WM_STYLECHANGING": 0x007C,
    "WM_THEMECHANGED": 0x031A,
    "WM_USERCHANGED": 0x0054,
    "WM_WINDOWPOSCHANGED": 0x0047,
    "WM_WINDOWPOSCHANGING": 0x0046
}


def send_message_to_window(window_name: str, action_message: int,
                           key_code_1: int, key_code_2: int):
    _hwnd = FindWindowW(window_name)
    post_status = SendMessageW(_hwnd, action_message, key_code_1, key_code_2)
    return _hwnd, post_status


def send_message_to_window_hwnd(_hwnd, action_message: int,
                                key_code_1: int, key_code_2: int):
    post_status = SendMessageW(_hwnd, action_message, key_code_1, key_code_2)
    return _hwnd, post_status


def post_message_to_window(window_name: str, action_message: int,
                           key_code_1: int, key_code_2: int):
    _hwnd = FindWindowW(window_name)
    post_status = PostMessageW(_hwnd, action_message, key_code_1, key_code_2)
    return _hwnd, post_status


def post_message_to_window_hwnd(_hwnd, action_message: int,
                                key_code_1: int, key_code_2: int):
    post_status = PostMessageW(_hwnd, action_message, key_code_1, key_code_2)
    return _hwnd, post_status



english_word_dict = {
    # Main
    "application_name": "AutoControlGUI",
    # Widget
    "interval_time": "Interval Time (s):",
    "cursor_x": "Cursor X Position:",
    "cursor_y": "Cursor Y Position:",
    "mouse_button": "Mouse Button:",
    "keyboard_button": "Keyboard Button:",
    "click_type": "Click Type:",
    "input_method": "Input Method:",
    "mouse_radio": "Mouse",
    "keyboard_radio": "Keyboard",
    "repeat_until_stopped_radio": "Repeat until stopped",
    "repeat_radio": "Repeat",
    "times": "Times",
    "start": "Start",
    "stop": "Stop",
}
